PROGRAM Check_Disk_For_Fragmentation;
{  List fragmented files via re-directed temp file of CHKDSK output.

1990/12/05 DWR * Finally ready to show, as v0r0m06.
1990/09/15 DWR * Begin R&D.
}

{$M $4000,$4000,$4000}   { 16K stack, 16K min heap, 16K max heap }

USES DOS, CRT;


CONST

   BackSpace    = #008;       { clear progress bar }
   OpenBlock    = #233;       { large progress symbol  }
   ClosedBlock  = #236;       { small progress symbol  }


VAR

   Input_Count  : Integer;    { current cycle count }
   Input_Visual : Integer;    { display @Nth cycle }
   Visual_Count : Integer;    { length of progress bar }
   Visual_Block : Char;       { current progress symbol }

   S              : Text;     { redirectable messages via STDOUT }
   IO_Status      : String;   { display version of I/O error code }
   IO_Status_X    : String;
   LST_File       : Text;     { final report vs STDOUT }
   LST_Name       : String;
   BAT_File       : Text;     { temp.BAT file of CHKDSK commands }
   BAT_Name       : String;
   BAT_Line       : String;
   CHK_File       : Text;     { temp.CHK file of CHKDSK output }
   CHK_Name       : String;
   CHK_Line       : String;
   User_SubDir    : String;   { return to currently logged D:\DIR }
   Temp_SubDir    : String;   { D:\DIR to hold temp files }
   Check_SubDir   : String;   { initial CHKDSK directory }
   Check_Pattern  : String;   { file pattern for CHKDSK to process }

   Dir_Count      : Word;     { total subdirectories searched }
   File_Count     : Word;     { total files (within search spec?) }
   Frag_Count     : Word;     { fragmented files found by CHKDSK }
   DOS_ErrorLevel : Word;     { ERRORLEVEL returned to DOS }

   Recurse_Sub_SW : Boolean;  { recurse into sub-directories switch }
   Chkdsk_Exec_SW : Boolean;  { execute temp.BAT switch }
   File_Count_SW  : Boolean;  { display file vs block frag count at eoj }
   Overwrite_SW   : Boolean;  { overwrite existing /FO file }
   Help_Short_SW  : Boolean;  { help usage switch }
   Help_Long_SW   : Boolean;  { help description switch }
   ValidInit_SW   : Boolean;  { abort if initialization errors }

   Hold_BAT_Name  : String;   { final name for temp.BAT }
   Hold_CHK_Name  : String;   { final name for temp.CHK }



FUNCTION IsConsole (VAR TextFile : TEXT) : Boolean;
{  Supplied by Philippe Ranger (TeamB) 71531,1350 via CI$ BPROGA.  It
   returns TRUE if the parameter is really the STDOUT console.  FALSE
   is returned if the parameter is a file or STDOUT redirected to a file.
}

VAR
   Regs : Registers;

BEGIN

   WITH Regs DO
      BEGIN
      AX := $4400;
      BX := TextRec(TextFile).Handle;
      MSDos(Regs);
      IF (DX AND $80) = 0
         THEN IsConsole := FALSE
         ELSE IsConsole := ((DX and $02) <> 0) or ((DX and $01) <> 0);
      END;

END; { IsConsole within Procss_Command_Line }



PROCEDURE Display_Help;
{  Display version and syntax.
}


PROCEDURE Display_Help_Page;   { sub-proc of Display_Help }
{  Provide crude paged display of help text.
}

VAR
   Response : Char;

BEGIN

   IF IsConsole(S)
      THEN BEGIN
         WriteLn(S,' ');
         WriteLn(S,'                             press any key to continue');
         Response := ReadKey;
         WriteLn(S,' ');
         END
      ELSE WriteLn(S,' ');

END; { Display_Help_Page within Display_Help }


BEGIN { Display_Help }

   WriteLn(S,'CKFRAG : list fragmented files');
   WriteLn(S,'         by Donald Randolph, December 1990 v0r0m06');
   IF Help_Long_SW THEN BEGIN
      WriteLn(S,' ');
      WriteLn(S,'   This utility creates a list of  fragmented files using');
      WriteLn(S,'CHKDSK as a  sub-task.   The DOS "errorlevel" is  used to');
      WriteLn(S,'return an error code (values 201 thru 255)  or as a count');
      WriteLn(S,'of files/blocks found fragmented.  A value of 200 doesn''t');
      WriteLn(S,'mean 200 frags, it only  indicates  more than  199 frags!');
      WriteLn(S,'   The error code could  let a batch file  determine if a');
      WriteLn(S,'de-fragger needs to be run.  Triple digit frag counts are');
      WriteLn(S,'probably  serious enough the  exact count is not  needed.');
      WriteLn(S,'Though any batch logic would have to know codes 201 and  ');
      WriteLn(S,'up mean the CKFRAG processing was not complete.          ');
      Display_Help_Page;
      WriteLn(S,'   When CHKDSK is given a file spec,  it generates a list');
      WriteLn(S,'of any matching files that are  fragmented.   But it does');
      WriteLn(S,'NOT  recurse  through  sub-directories for a frag list of');
      WriteLn(S,'the entire drive.  CKFRAG started as a exercise in Pascal');
      WriteLn(S,'recursion to build a  batch file of a  drives tree.  This');
      WriteLn(S,'batch file issues  "CHKDSK *.*" for  each directory,  re-');
      WriteLn(S,'directing the CHKDSK output to a  text file.  CKFRAG then');
      WriteLn(S,'reads that text,  extracts the  fragmented  file names to');
      WriteLn(S,'the screen, and displays counts of files & total blocks.');
      WriteLn(S,'   Since CKFRAG spawns a child to run re-directed CHKDSK,');
      WriteLn(S,'CKFRAG  itself can  not  be run with  re-direction.  This');
      WriteLn(S,'confuses  CHKDSK  into  thinking you have  lost  clusters');
      WriteLn(S,'because DOS has opened  nested  temporary files.   CKFRAG');
      WriteLn(S,'detects  first-level  re-direction and will  exit with an');
      WriteLn(S,'error  instead of trying to run CHKDSK.   This might have');
      WriteLn(S,'meant you were  SOL if the file name list  scrolled.  But');
      WriteLn(S,'there is a switch to tell CKFRAG to  write the names to a');
      WriteLn(S,'text file instead of the screen.                         ');
      Display_Help_Page;
      WriteLn(S,'   The defaults for CKFRAG will search for any fragmented');
      WriteLn(S,'file in the  current directory and  its  sub-directories,');
      WriteLn(S,'build temporary work files in the current directory  (and');
      WriteLn(S,'delete them when finished),  display frag report.  If the');
      WriteLn(S,'first command line parameter is not switch (doesn''t start');
      WriteLn(S,'with a slash)  it is assumed  to be  a drive  and/or path');
      WriteLn(S,'and/or file spec :');
      WriteLn(S,'      d:          search *.* from logged directory');
      WriteLn(S,'      d:\dir\     search *.* from \DIR');
      WriteLn(S,'                  closing backslash indicates dir vs file');
      WriteLn(S,'      d:\dir\*.*  search file spec from \DIR');
      WriteLn(S,'      *.*         search file spec from logged directory');
      Display_Help_Page;
      END;
   WriteLn(S,' usage : CKFRAG [drive:][path][filespec] [switches]');
   WriteLn(S,' /rs+ Recurse into Subdirectories, +/-');
   WriteLn(S,' /ck+ actually shell to execute temp.BAT, +/-');
   WriteLn(S,' /fc+ return code as file vs block fragmentation count, +/-');
   WriteLn(S,' /lsX list fragmentation report to X=filename vs STDOUT');
   WriteLn(S,' /poX build temp files in X=D:\PATH');
   WriteLn(S,' /foX save temp.BAT as X=filename');
   WriteLn(S,' /coX save temp.CHK as X=filename');
   WriteLn(S,' /ow- OverWrite existing list/save files, +/-');
   WriteLn(S,' /h   short help screen');
   WriteLn(S,' /hh  long help screen');

END; { Display_Help }



PROCEDURE Display_Visual_Progress;
{  Roll quintuplet of alternating graphics to indicate progress.
}

BEGIN

   INC(Visual_Count);
   Write(S,Visual_Block);
   IF Visual_Count = 5 THEN
      BEGIN
      Visual_Count := 0;
      Write(S,BackSpace + BackSpace + BackSpace + BackSpace + BackSpace);
      IF Visual_Block = OpenBlock
         THEN Visual_Block := ClosedBlock
         ELSE Visual_Block := OpenBlock;
      END;
   Input_Count := 0;

END; { Display_Visual_Progress }



PROCEDURE Display_Error (Error_Code : Integer; Error_Msg : String);
{  Set code for batch errorlevel, beep and display message.
}

BEGIN

   ValidInit_SW := false;
   {IF DOS_ErrorLevel < Error_Code THEN} DOS_ErrorLevel := Error_Code;
   Sound(3000); Delay(100); Nosound;
   WriteLn(S, Error_Code, ' : ', Error_Msg);

END; { Display_Error }



PROCEDURE Process_Command_Line;
{  Extract file names and switches from command line.  Sub-procedures are
   used to set switch values and the output file name.  Open files with
   error checking.
}

VAR

   Parm_Str  : String;
   Parm_Id   : Integer;
   Dir_Info  : SearchRec;

   ChkPath   : PathStr;
   ChkDir    : DirStr;
   ChkName   : NameStr;
   ChkExt    : ExtStr;



PROCEDURE Determine_Option;   { sub-proc of Process_Command_line }
{  Set switch values from command line.  Assume default if switch value is
   not opposite flag.  Declare initialization error if unknown switch name.
}

VAR

   Parm_Type : String;

BEGIN

   Parm_Type := Copy(Parm_Str,2,2);
   IF Parm_Type = 'rs' THEN
      IF Parm_Str[4] = '-'
         THEN BEGIN Recurse_Sub_SW := false; Exit; END
         ELSE BEGIN Recurse_Sub_SW := true;  Exit; END;
   IF Parm_Type = 'ck' THEN
      IF Parm_Str[4] = '-'
         THEN BEGIN ChkdsK_Exec_SW := false; Exit; END
         ELSE BEGIN ChkdsK_Exec_SW := true;  Exit; END;
   IF Parm_Type = 'fc' THEN
      IF Parm_Str[4] = '-'
         THEN BEGIN File_Count_SW  := false; Exit; END
         ELSE BEGIN File_Count_SW  := true;  Exit; END;
   IF Parm_Type = 'ow' THEN
      IF Parm_Str[4] = '+'
         THEN BEGIN Overwrite_SW   := true;  Exit; END
         ELSE BEGIN Overwrite_SW   := false; Exit; END;
   IF Parm_Type = 'ls' THEN BEGIN
      LST_Name := Copy(Parm_Str,4,Length(Parm_Str)-3);
      Exit; END;
   IF Parm_Type = 'fo' THEN BEGIN
      Hold_BAT_Name := Copy(Parm_Str,4,Length(Parm_Str)-3);
      Exit; END;
   IF Parm_Type = 'co' THEN BEGIN
      Hold_CHK_Name := Copy(Parm_Str,4,Length(Parm_Str)-3);
      Exit; END;
   IF Parm_Type = 'po' THEN BEGIN
      Temp_SubDir := Copy(Parm_Str,4,Length(Parm_Str)-3);
      Exit; END;
   IF Parm_Type = 'h'  THEN BEGIN
      Help_Short_SW := true;
      Exit; END;
   IF Parm_Type = 'hh' THEN BEGIN
      Help_Long_SW := true;
      Exit; END;
   Display_Error(204, 'unknown switch ' + Parm_Str);

END; { Determine_Option within Process_Command_Line }



FUNCTION Open_Unique_File (SPath:String; Ext:String) :String;
{  Copied from CI$ BPROGA, from Paul Willmott 72510,3526 to Chris Nelson
   70441,3321 with additional comments by Kim K. (TurboPower) 76004,2611.

   This version accepts a string of the D:\DIR path the new file should
   reside in, obtains a unique name from DOS, renames it with the supplied
   extension and returns the new name.  If the returned string begins with
   a pound sign, it is really the DOS error code.
}

VAR
   Unique_File : Text;
   Unique_Reg  : Registers;
   Unique_Name : String;

BEGIN
   Open_Unique_File := '#??'; { assume error flag }
   Unique_Name := SPath + #000;  { convert path name to ASCIIZ }
   Unique_Reg.AH := $5A;         { DOS function to open unique name }
   Unique_Reg.DS := Seg(Unique_Name[1]);
   Unique_Reg.DX := Ofs(Unique_Name[1]);
   Unique_Reg.CX := $00;
   MsDos(Unique_Reg);            { do it }
   IF (Unique_Reg.Flags AND FCarry = FCarry)
      THEN BEGIN
         Str(Unique_Reg.AX:4, Unique_Name);
         Open_Unique_File := '#A-' + Unique_Name;
         END
      ELSE BEGIN
         Unique_Name[0] := #255; { max Pascal string length for search }
         Unique_Name[0] := Chr(Pred(Pos(#000,Unique_Name))); { undo ASCIIZ }
         Unique_Reg.BX := Unique_Reg.AX; { BX is handle to be closed }
         Unique_Reg.AH := $3E;   { close file, releasing handle }
         MsDos(Unique_Reg);
         IF (Unique_Reg.Flags AND FCarry = FCarry)
            THEN BEGIN
               Str(Unique_Reg.AX:4, Unique_Name);
               Open_Unique_File := '#B-' + Unique_Name;
               END
            ELSE BEGIN
               Assign(Unique_File, Unique_Name);
               {$I-} Rename(Unique_File, Unique_Name + '.' + Ext); {$I+}
               Str(IOResult, IO_Status);
               IF IO_Status = '0'
                  THEN Open_Unique_File := Unique_Name + '.' + Ext
                  ELSE Open_Unique_File := '#C-' + IO_Status;
               END;
         END ;

END; { Open_Unique_File within Process_Command_Line }



PROCEDURE Test_For_OverWrite (File_Name : String);
   { sub-proc of Process_Command_line }
{  Turn off ValidInit_SW if file exists and /OW+ was not set.
}

VAR

   F       : Text;
   Last_IO : String;

BEGIN

   Last_IO := '0';
   {$I-}
   Assign(F, File_Name);
   Reset(F);
   Close(F);
   {$I+}
   Str(IOResult, IO_Status);
   IF (IO_Status = '0') AND (NOT Overwrite_SW)
      THEN Display_Error(207, 'file ' + File_Name +
               ' already exists - use /ow+');

END; { Test_For_OverWrite within Process_Command_Line }



BEGIN { Process_Command_Line MAIN of procedure set }

   { Initialize variables.  The default switch values must stay snychronzied
     with the Determine_Option sub-procedure.
   }
   Check_SubDir   := '.';   { current sub-dir }
   Check_Pattern  := '*.*'; { all files }
   Chkdsk_Exec_SW := true;
   Recurse_Sub_SW := true;
   File_Count_SW  := true;
   Overwrite_SW   := false;
   Help_Short_SW  := false;
   Help_Long_SW   := false;
   ValidInit_SW   := true;
   Hold_CHK_Name  := '';
   Hold_BAT_Name  := '';
   LST_Name       := '';
   BAT_Name       := '';
   CHK_Name       := '';
   Temp_SubDir    := '';
   Visual_Count   := 0;
   Visual_Block   := OpenBlock;
   Input_Visual   := 10;
   Frag_Count     := 0;
   File_Count     := 0;
   Dir_Count      := 0;
   DOS_ErrorLevel := 0;

   {$I-} GetDir(0,User_SubDir); {$I+} { save currently logged sub-dir }
   Str(IOResult, IO_Status);
   IF IO_Status <> '0'
      THEN Display_Error(210, 'cannot find logged sub-dir - error '
               + IO_Status)
      ELSE Check_Subdir := User_Subdir; {default to current subdir }

   IF ParamCount <> 0 THEN
      BEGIN
      ChkPath := ParamStr(1); { is there a user dir/file spec ? }
      IF ChkPath[1] <> '/'
         THEN BEGIN { parm1 not a switch, must be file spec }
            FSplit(ChkPath, ChkDir, ChkName, ChkExt); { split dir/file }
            IF ChkDir <> '' THEN Check_Subdir := Copy(ChkDir,1,Length(ChkDir)-1);
            IF ChkName = '' THEN ChkName := '*'; { insure valid file spec }
            IF ChkExt  = '' THEN ChkExt  := '.*';
            Check_Pattern := ChkName + ChkExt;
            IF ChkDir[2] = ':' THEN ChkDir := Copy(ChkDir,1,3); { verify drive }
            END
         ELSE BEGIN
            ChkDir := ''; { nul drive for initial chkdsk verify }
            END;
      FOR Parm_Id := 1 to ParamCount DO { look for command line switches }
         BEGIN
         Parm_Str := ParamStr(Parm_Id);
         IF Parm_Str[1] = '/' THEN Determine_Option;
         END;
      END;

   {$I-} ChDir(Check_Subdir); {$I+}  { verify valid subdirectory }
   Str(IOResult, IO_Status);
   IF IO_Status <> '0'
      THEN Display_Error(213, 'cannot find sub-dir ' + Check_Subdir
			         + ' - error ' + IO_Status);
   {$I-} ChDir(User_SubDir); {$I+}  { return to original subdirectory }
   Str(IOResult, IO_Status);
   IF IO_Status <> '0'
      THEN Display_Error(216, 'cannot return to sub-dir ' + User_Subdir
			         + ' - error ' + IO_Status);

   IF Help_Short_SW or Help_Long_SW or not ValidInit_SW
      THEN Exit; { skip file opens }

   IF NOT IsConsole(S)
      THEN Display_Error(219, 'can not run with STDOUT redirected,'
               + 'use /lsX switch');

   IF Temp_SubDir = '' THEN Temp_SubDir := User_SubDir;
   IF BAT_Name = '' THEN
      BEGIN
      BAT_Name := Open_Unique_File(Temp_SubDir, 'bat');
      IF BAT_Name[1] = '#'
         THEN Display_Error(222, 'error allocating BAT file - ' + BAT_Name);
      END;

   CHK_Name := Open_Unique_File(Temp_SubDir, 'chk');
   IF CHK_Name[1] = '#'
      THEN Display_Error(225, 'error allocating CHK file - ' + CHK_Name);

   Assign(BAT_File, BAT_Name);
   {$I-} Rewrite(BAT_File); {$I+}
   Str(IOResult, IO_Status);
   IF IO_Status <> '0'
      THEN Display_Error(228, 'error ' + IO_Status + ' reusing ' + BAT_Name);

   IF Hold_CHK_Name <> '' THEN Test_For_OverWrite(Hold_CHK_Name);
   IF Hold_BAT_Name <> '' THEN Test_For_OverWrite(Hold_BAT_Name);
   IF      LST_Name <> '' THEN Test_For_OverWrite(LST_Name);

   IF ValidInit_SW THEN
      BEGIN
      WriteLn(S,' ');
      WriteLn(S,'shell to verify disk structure');
      SwapVectors;
      Exec(FSearch('chkdsk.com',GetEnv('path')), ChkDir);
      SwapVectors;
      IF DosError = 0
         THEN BEGIN
            IF DosExitCode <> 0 THEN
               BEGIN
               Str(Hi(DosExitCode), IO_Status);
               Str(Lo(DosExitCode), IO_Status_X);
               Display_Error(231, 'verify CHKDSK terminated by '
                     + IO_Status + ' with errorlevel ' + IO_Status_X);
               END;
            END
         ELSE BEGIN
            Str(DosError, IO_Status);
            Display_Error(234, 'verify shell error #' + IO_Status);
            END;
      END;

END; { Process_Command_Line MAIN of procedure set }



PROCEDURE Rename_Work_File(Work_Name, New_Name: String);
{  Rename work file from unique DOS generated value, per users request.  If
   the users hold name does not specify a path, use same as work path.  The
   work path might have an ending backslash if the user specified it via
   the /po switch.
}

VAR

   Work_File : Text;
   NewPath   : PathStr;
   NewDir    : DirStr;
   NewName   : NameStr;
   NewExt    : ExtStr;

BEGIN

   NewPath := New_Name; { did user specify path on hold file option? }
   FSplit(NewPath, NewDir, NewName, NewExt);
   IF NewDir = '' THEN
      IF Temp_Subdir[Length(Temp_Subdir)] = '\'
         THEN New_Name := Temp_Subdir       + NewName + NewExt
         ELSE New_Name := Temp_Subdir + '\' + NewName + NewExt;

   IF Overwrite_SW THEN
      BEGIN
      Assign(Work_File, New_Name);
      {$I-} Erase(Work_File) {$I+};
      Str(IOResult, IO_Status);  { dummy to swallow IO code }
      END;
   Assign(Work_File, Work_Name);
   {$I-} Rename(Work_File, New_Name) {$I+};
   Str(IOResult, IO_Status);
   IF IO_Status <> '0'
      THEN Display_Error(237, 'error ' + IO_Status + ' renaming '
               + Work_Name + ' to ' + New_Name);

END; { Rename_Work_File }



PROCEDURE Delete_Work_File(Work_Name: String);
{  Delete work file from unique DOS generated value.
}

VAR

   Work_File : Text;

BEGIN

   Assign(Work_File, Work_Name);
   {$I-} Erase(Work_File); {$I+}
   Str(IOResult, IO_Status);
   IF IO_Status <> '0'
      THEN Display_Error(240, 'error ' + IO_Status + ' deleting '
               + Work_Name);

END; { Delete_Work_File }



Procedure Recursive_Search(Current_Subdir : String);
{  Recurse through subdirectories and count matching file specs.
}

VAR
	 Search_Subdir  : String;
	 Search_DTA     : SearchRec;
	 Recurse_Subdir : String;
    Passed_Subdir  : String;

BEGIN

   Display_Visual_Progress;
   INC(Dir_Count);
   IF Current_Subdir[Length(Current_Subdir)] = '\' { normalize path }
      THEN BEGIN
         Search_Subdir  := Current_Subdir + '*.*';
         Recurse_Subdir := Current_Subdir;
         END
      ELSE BEGIN
         Search_Subdir  := Current_Subdir + '\*.*';
         Recurse_Subdir := Current_Subdir + '\';
         END;

   Str(Dir_Count, IO_Status);
   WriteLn(BAT_File,'echo #' + IO_Status + '  ' + Recurse_Subdir);
   WriteLn(BAT_File,'echo #' + IO_Status + '  ' + Recurse_Subdir
                         + ' >> ' + CHK_Name);
   WriteLn(BAT_File,'chkdsk ' + Recurse_Subdir
         + Check_Pattern + ' >> ' + CHK_Name);

   IF Recurse_Sub_SW THEN     { recurse into subdirectories }
      BEGIN FindFirst(Search_Subdir,Directory,Search_DTA);
      WHILE (DOSError <> 2) AND (DOSError <> 18) DO
         BEGIN                { until no more files found }
         IF ((Search_DTA.Attr AND Directory) = Directory) { really subdir ? }
         AND (Search_DTA.Name[1] <> '.') THEN { but not current/parent }
            BEGIN
            Passed_Subdir := Recurse_Subdir + Search_DTA.Name;
            Recursive_Search(Passed_Subdir);
            END;
         FindNext(Search_DTA);
         END;
      END;

   IF Current_Subdir[Length(Current_Subdir)] = '\' { normalize path }
      THEN Search_Subdir  := Current_Subdir + Check_Pattern
      ELSE Search_Subdir  := Current_Subdir + '\' + Check_Pattern;
   FindFirst(Search_Subdir,AnyFile,Search_DTA); { begin counting files }
   WHILE DOSError <> 18 DO { until no more files found }
      BEGIN
      INC(File_Count);
      FindNext(Search_DTA);
      END;

END; { Recursive_Search }



PROCEDURE Relist_Chkdsk_Output;
{  Display and count file names piped from CHKDSK.
}

VAR
   Report_Ptr : ^Text;
   Val_Block  : Integer;
   Str_Begin  : Integer;
   Str_Length : Integer;
   Str_Block  : String[10];

BEGIN

   Assign(CHK_File, CHK_Name); { open CHKDSK reports for input }
   Reset(CHK_File);
   IF LST_Name <> ''
      THEN BEGIN
         Assign(LST_File, LST_Name);
         Rewrite(LST_File);
         Report_Ptr := @LST_File;
         END
      ELSE Report_Ptr := @S;

   WHILE NOT Eof(CHK_File) DO
      BEGIN
      ReadLn(CHK_File, CHK_Line);
      IF CHK_Line[2] = ':' THEN  { display fragmented file names }
         BEGIN
         WriteLn(Report_Ptr^, CHK_Line);
         INC(File_Count);
         Str_Begin  := Pos('Contains ', CHK_Line) + 9;
         Str_Length := Pos(' non-cont', CHK_Line) - Str_Begin;
         Str_Block  := Copy(CHK_Line, Str_Begin, Str_Length);
         Val(Str_Block, Val_Block, Str_Begin);
         Frag_Count := Frag_Count + Val_Block;
         END;
      END;

   IF LST_Name <> '' THEN Close(LST_File);
   Close(CHK_File);

END; { Relist_Chkdsk_Output }



BEGIN { PROGRAM Check_Disk_For_Fragmentation }

   Assign(S,'');              { console messages to STDOUT }
   Rewrite(S);
   Process_Command_Line;      { parse command line parameters }
   IF Help_Short_SW or Help_Long_SW THEN
      BEGIN
      Display_Help;
      Exit;
      END;

   IF NOT ValidInit_SW THEN
      BEGIN
      Close(S);
      Halt(DOS_ErrorLevel);
      END;

   Write(S,'building CHKDSK command file:');
   WriteLn(BAT_File, 'echo off');
   WriteLn(BAT_File, 'echo CKFRAG temp ', BAT_Name, '  (', Hold_BAT_Name, ')');
   WriteLn(BAT_File, 'echo             ', CHK_Name, '  (', Hold_CHK_Name, ')');
   WriteLn(BAT_File, 'echo initialize ', CHK_Name, ' > ', CHK_Name);

   Recursive_Search(Check_Subdir); { count matching file spec & recurse subdir }

   Close(BAT_File);
   Str(Dir_Count, IO_Status); { display process counts }
   Write(S,'  sub-dirs=' + IO_Status);
   Str(File_Count, IO_Status);
   WriteLn(S, '   files=' + IO_Status);
   File_Count := 0;           { reset for fragmented vs total count }

   IF ValidInit_SW THEN
   BEGIN
      IF Chkdsk_Exec_SW THEN
         BEGIN
         WriteLn(S,' ');
         WriteLn(S,'shell to run batch command file - do NOT interrupt');
         SwapVectors;
         Exec(GetEnv('comspec'), ' /c ' + BAT_Name);
         SwapVectors;
         WriteLn(S,'batch command file complete');
         IF DosError = 0
         THEN BEGIN
            IF DosExitCode <> 0 THEN
               BEGIN
               Str(Hi(DosExitCode), IO_Status);
               Str(Lo(DosExitCode), IO_Status_X);
               Display_Error(243, 'CHKDSK command terminated by ' + IO_Status
                     + ' with errorlevel ' + IO_Status_X);
               END;
            WriteLn(S,' ');
            WriteLn(S,'searching for fragmented file names from CHKDSK');
            Relist_Chkdsk_Output;
            END
         ELSE BEGIN
            Str(DosError, IO_Status);
            Display_Error(247, 'search shell error #' + IO_Status);
            END;
         END;
   IF Hold_BAT_Name <> ''
      THEN Rename_Work_File(BAT_Name, Hold_BAT_Name)
      ELSE Delete_Work_File(BAT_Name);
   IF Hold_CHK_Name <> ''
      THEN Rename_Work_File(CHK_Name, Hold_CHK_Name)
      ELSE Delete_Work_File(CHK_Name);
   END;

   WriteLn(S,'fragmentation file  count = ', File_Count);
   WriteLn(S,'fragmentation block count = ', Frag_Count);
   IF DOS_ErrorLevel = 0
      THEN IF Frag_Count > 200
         THEN DOS_ErrorLevel := 200
         ELSE IF File_Count_SW
            THEN DOS_ErrorLevel := File_Count
            ELSE DOS_ErrorLevel := Frag_Count;
   Close(S);
   Halt(DOS_ErrorLevel);

END. { PROGRAM Check_Disk_For_Fragmentation }
